/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 *
 */

package net.dries007.tfc.api;

import java.util.Random;

import net.minecraft.block.BlockSapling;
import net.minecraft.util.Rotation;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraft.world.gen.structure.template.PlacementSettings;
import net.minecraft.world.gen.structure.template.TemplateManager;

import net.dries007.tfc.api.types.Tree;

public interface ITreeGenerator
{

    /**
     * Called to generate a tree. Each Tree must have one of these. Used for world gen and sapling growth
     *
     * @param manager an instance of the world's template manager. Used for getting structures.
     * @param world   The world
     * @param pos     The position where the sapling was / would've been
     * @param tree    The tree type to spawn
     * @param rand    A random to use in generation
     */
    void generateTree(TemplateManager manager, World world, BlockPos pos, Tree tree, Random rand);

    /**
     * @return A default set of placement settings for tree generation
     */

    static PlacementSettings getDefaultSettings()
    {
        return new PlacementSettings()
            .func_186222_a(true)
            .func_186226_b(false);
    }

    /**
     * @param rand For generating random settings
     * @return A set of placement settings with random rotation
     */
    static PlacementSettings getRandomSettings(Random rand)
    {
        return getDefaultSettings()
            .func_186220_a(Rotation.values()[rand.nextInt(Rotation.values().length)]);
    }

    static boolean checkGenerationConditions(World world, BlockPos pos, Tree treeType)
    {
        // Check if ground is flat enough
        final int radius = treeType.maxGrowthRadius;
        for (int x = -radius; x <= radius; x++)
        {
            for (int z = -radius; z <= radius; z++)
            {
                if ((x == 0 && z == 0) ||
                    world.func_180495_p(pos.func_177982_a(x, 0, z)).func_185904_a().func_76222_j() ||
                    ((x > 1 || z > 1) && world.func_180495_p(pos.func_177982_a(x, 1, z)).func_185904_a().func_76222_j()))
                    continue;
                return false;
            }
        }
        // Check if there is room directly upwards
        final int height = treeType.maxHeight;
        for (int y = 1; y <= height; y++)
            if (!world.func_180495_p(pos.func_177981_b(y)).func_185904_a().func_76222_j())
                return false;

        // Check if there is a solid block beneath
        if (!world.func_180495_p(pos.func_177977_b()).func_185915_l())
            return false;

        // Check the position for liquids, etc.
        if (world.func_180495_p(pos).func_185904_a().func_76224_d() || !world.func_180495_p(pos).func_185904_a().func_76222_j())
            if (!(world.func_180495_p(pos) instanceof BlockSapling))
                return false;

        // Check if there is sufficient light level
        return world.func_175671_l(pos) >= 7;
    }

    /**
     * Checks if a tree can be generated. This implementation checks height, radius, and light level
     * Suggested use is to use within your implementation of generateTree()
     *
     * @param world    The world
     * @param pos      The pos of the tree
     * @param treeType The tree type (for checking if the tree can generate)
     * @return true if the tree can generate.
     */
    default boolean canGenerateTree(World world, BlockPos pos, Tree treeType)
    {
        return checkGenerationConditions(world, pos, treeType);
    }
}
